package riomhaire.lti.business.actions;

import lombok.Builder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.servlet.ModelAndView;
import riomhaire.lti.infrastructure.Registry;
import riomhaire.lti.model.interfaces.Action;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Function;

@Slf4j
@Builder
/**
 * LaunchContent is called after we have decoded the ltu1.3 token an ascertained that the message type is 'LtiResourceLinkRequest'
 * This simple example just displays whats in the message after simplifies the main keys
 */
public class LaunchContent implements Action<ModelAndView> {
    Registry registry;
    Map<String, String> queryParams;
    Map<String, Object> metadata;
    Map<String, Object> claims;


    @Override
    public ModelAndView execute() {
        final String prefix = "https://purl.imsglobal.org/spec/lti/claim/";
        var simplifiedClaims = new HashMap<String,Object>();
        var mav = new ModelAndView("dump-launch");
//        var mav = new ModelAndView("redirect:https://www.riomhaire.com");

        // Strip "https://purl.imsglobal.org/spec/lti/claim/" from keys
        Function<String,String> simplifier = s -> {
                if (s != null && s.startsWith(prefix)){
                    return s.substring(prefix.length());
                }
                return s;
        };
        claims.forEach((s, o) -> simplifiedClaims.put(simplifier.apply(s),o));
        mav.addObject("claims",simplifiedClaims);
        mav.addObject("metadata",metadata);

        return mav;
    }
}
