package riomhaire.lti.business.actions;

import lombok.Builder;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.view.RedirectView;
import org.springframework.web.servlet.view.json.MappingJackson2JsonView;
import riomhaire.lti.infrastructure.Registry;
import riomhaire.lti.model.interfaces.Action;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@Slf4j
@Builder
/***
 *  ProcessOidcRequest handles the oidc initiation step of the oidc flow ...
 *  you should add some checks and generate state value you can check on in later calls.
 *
 *  It uses a client registry (in this implementation based on a json object in redis) to locate where
 *  the authenticate endpoint is locate.
 */
public class ProcessOidcRequest implements Action<ModelAndView> {
    Registry registry;
    Map<String, String> queryParams;
    Map<String, Object> metadata;

    /**
     * Executte
     * @return
     */
    public ModelAndView execute() {
        final ModelAndView mav;
        String iss = queryParams.get("iss");
        String clientId = queryParams.get("client_id");
        var clientRegistration = registry.clientRegistrationResolver().lookupClient(iss, clientId);
        var claims = new HashMap<String,Object>();

        if (clientRegistration.isPresent()) {
            // Build Location
            var location = new StringBuffer();
            var clientConfiguration = clientRegistration.get();
            var loginHint = queryParams.get("login_hint");
            var ltiMessageHint = queryParams.get("lti_message_hint");
            var targetLinkUri = queryParams.get("target_link_uri");


            location.append(clientConfiguration.getAuthenticateUrl());
            location.append("?");
            location.append("scope=openid&response_type=id_token&client_id=");
            location.append(clientConfiguration.getClientId());
            location.append("&login_hint=");
            location.append(loginHint);

            // This should be used to map to some backend thing we can verify ... jwt etc
            location.append("&state=");
            final String state = UUID.randomUUID().toString();
            location.append(state);

            // This should be used to map to some backend thing we can verify ... as once only
            location.append("&response_mode=form_post&prompt=none&nonce=");
            location.append(UUID.randomUUID().toString());
            location.append("&lti_message_hint=");
            location.append(ltiMessageHint);
            location.append("&redirect_uri=");
            location.append(targetLinkUri);

            log.info("OIDC Redirecting to "+location.toString());
            mav = new ModelAndView(new RedirectView(location.toString()));

        } else {
            log.error("unknown issuer ["+iss+"] client ["+clientId+"]");
            mav = new ModelAndView(new MappingJackson2JsonView());
            mav.addObject("bean", claims);
        }
        return mav;
    }
}
